/*
Switch control LED
Device: AI8051U
Author: Jin-Lei Li
Email: lijinlei0907@163.com
Date: Nov. 11, 2024
*/

#include <STC8H.H>

typedef unsigned char u8;
typedef unsigned int u16;
typedef unsigned long int u32;

#define MAIN_Fosc 24000000UL // main frequency

sbit Key_Up = P7^4;
sbit Key_Down = P7^3;
sbit Key_Left = P3^2;  // Fancy LED
sbit Key_Right = P7^1;
sbit Key_Middle = P7^0;
sbit Key_Esc = P4^7; // forward LED
sbit Key_On = P3^3; // reverse LED
sbit Key_Flow = P5^3; // Flow LED include forward and reverse LED

u8 LED_Code[8] = {0xFE,0xFD,0xFB,0xF7,0xEF,0xDF,0xBF,0x7F};

void delayms(u16 ms);
void KeyScan();

//u8 KeyNumber;

void main()
{
	P1M1 = 0x00;P1M0 = 0x00;
	P2M1 = 0x00;P2M0 = 0x00;
	P3M1 = 0x00;P3M0 = 0x00;
	P4M1 = 0x00;P4M0 = 0x00;
	P5M1 = 0x00;P5M0 = 0x00;
	P6M1 = 0x00;P6M0 = 0x00;
	P7M1 = 0x00;P7M0 = 0x00;
	
	while(1)
	{
		KeyScan();
	}
}

void KeyScan()
{
	u8 i;
	u8 sum1 = 0;
	u8 sum2 = 0;
	static u8 KeyState_Up, KeyState_Down, KeyState_Left, KeyState_Right, KeyState_Middle, KeyState_Esc, KeyState_On, KeyState_Flow;
	if(Key_Up == 0 && KeyState_Up == 0)
	{
		delayms(10);
		if(Key_Up == 0 && KeyState_Up == 0)
		{
			KeyState_Up = 1;
			P2 = LED_Code[1];
		}
	}
	if(Key_Up == 1)
	{
		KeyState_Up = 0;
		//P2 = 0xFF;
	}
	//--------------------------------------------
	if(Key_Down == 0 && KeyState_Down == 0)
	{
		delayms(10);
		if(Key_Down == 0 && KeyState_Down == 0)
		{
			KeyState_Down = 1;
			P2 = LED_Code[2];
		}
	}
	if(Key_Down == 1) 
	{
		KeyState_Down = 0;
		//P2 = 0xFF;
	}
	//-------------------------------------------
	if(Key_Left == 0 && KeyState_Left == 0)
	{
		delayms(10);
		if(Key_Left == 0 && KeyState_Left == 0)
		{
			// = 1;
			//P2 = LED_Code[3];
			for(i = 0;i <4;i++)
			{
				sum1 += ~LED_Code[2*i+1];
				sum2 += ~LED_Code[2*i];
			}
			i=0;
			P2 = ~sum1;
			delayms(300);
			P2 = ~sum2;
			delayms(300);
			i++;
		}
	}
	if(Key_Left == 1)
	{
		KeyState_Left = 0;
		//P2 = 0xFF;
	}
	//----------------------------------------
	if(Key_Right == 0 && KeyState_Right == 0)
	{
		delayms(10);
		if(Key_Right == 0 && KeyState_Right == 0)
		{
			KeyState_Right = 1;
			P2 = LED_Code[4];
		}
	}
	if(Key_Right == 1)
	{
		KeyState_Right = 0;
		//P2 = 0xFF;
	}
	//--------------------------------------
	if(Key_Middle == 0 && KeyState_Middle == 0)
	{
		delayms(10);
		if(Key_Middle == 0 && KeyState_Middle == 0)
		{
			KeyState_Middle = 1;
			P2 = LED_Code[5];
		}
	}
	if(Key_Middle == 1)
	{
		KeyState_Middle = 0;
		//P2 = 0xFF;
	}
	//-------------- forward LED ------------------------
	if(Key_Esc == 0 && KeyState_Esc == 0)
	{
		delayms(10);
		if(Key_Esc == 0 && KeyState_Esc == 0)
		{
			//KeyState_Esc = 1;
			//P2 = 0xFF;
			for(i = 0;i < 8;i++)
			{
				P2 = LED_Code[i];//~(0x01 << i); // forward LED
				delayms(100);
			}
		}
		if(Key_Esc == 1)
		{
		KeyState_Esc = 0;
		//P2 = 0xFF;
		}
	}
	//--------------- reverse all LED -----------------------
	if(Key_On == 0 && KeyState_On == 0)
	{
		delayms(10);
		if(Key_On == 0 && KeyState_On == 0)
		{
			//KeyState_On = 1;
			//P2 = 0x00;
			for(i = 0;i < 8;i++)
			{
				P2 = LED_Code[7-i];//~(0x80 >> i); // reverse LED
				delayms(100);
			}
		}
		if(Key_On == 1)
		{
			KeyState_On = 0;
			//P2 = 0xFF;
		}
	}
	//---------------Flow LED-----------------------
	if(Key_Flow == 0 && KeyState_Flow == 0)
	{
		delayms(10);
		if(Key_Flow == 0 && KeyState_Flow == 0)
		{
			KeyState_Flow = 1;
			for(i = 0;i < 8;i++)
			{
				P2 = LED_Code[i];//~(0x01 << i); // forward LED
				delayms(100);
			}
			for(i = 0;i < 8;i++)
			{
				P2 = LED_Code[7-i];//~(0x80 >> i); // reverse LED
				delayms(100);
			}
		}
		if(Key_Flow == 1)
		{
			KeyState_Flow = 0;
		}
	}
}

void delayms(u16 ms)
{
	u16 i;
	do
	{
		i = MAIN_Fosc / 6000;
		while(--i);
	}
	while(--ms);
}